@@ -1055,7 +1055,7 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv, i
 				break;
 
 			case 'l':
-				*(long *) loc = strtoul(input, &input, 0);
+				*(tgt_long *) loc = tgt_strtoul(input, &input, 0);
 				loc += tgt_sizeof_long;
 				break;
 
