@@ -146,6 +146,9 @@ arch_apply_relocation (struct obj_file *f,
 
     case R_390_PLT32:
     case R_390_PLT16DBL:
+      /* For PLT relocs, the addend applies to the PLT slot.  */
+      v -= rel->r_addend;
+
       /* find the plt entry and initialize it.  */
       assert(isym != NULL);
       pe = (struct s390_plt_entry *) &isym->pltent;
@@ -154,19 +157,19 @@ arch_apply_relocation (struct obj_file *f,
         unsigned int *ip = (unsigned int *)(ifile->plt->contents + pe->offset); 
         ip[0] = 0x0d105810; /* basr 1,0; lg 1,10(1); br 1 */
         ip[1] = 0x100607f1;
-       if (ELF32_R_TYPE(rel->r_info) == R_390_PLT16DBL)
-         ip[2] = v - 2;
-       else
-         ip[2] = v;
+	ip[2] = v;
         pe->initialized = 1;
       }
 
       /* Insert relative distance to target.  */
       v = plt + pe->offset - dot;
+      /* Add PLT slot addend here.  */
+      v += rel->r_addend;
+
       if (ELF32_R_TYPE(rel->r_info) == R_390_PLT32)
         *(unsigned int *) loc = (unsigned int) v;
       else if (ELF32_R_TYPE(rel->r_info) == R_390_PLT16DBL)
-        *(unsigned short *) loc = (unsigned short) ((v + 2) >> 1);
+        *(unsigned short *) loc = (unsigned short) (v >> 1);
       break;
 
     case R_390_GLOB_DAT:
